@@ -55,10 +55,20 @@ def find_dists(dists):
 
 def skip_upload(response, skip_existing, package):
     filename = package.basefilename
-    msg = 'A file named "{0}" already exists for'.format(filename)
-    return (response.status_code == 400 and
-            response.reason.startswith(msg) and
-            skip_existing)
+    # NOTE(sigmavirus24): Old PyPI returns the first message while Warehouse
+    # returns the latter. This papers over the differences.
+    msg = ('A file named "{0}" already exists for'.format(filename),
+           'File already exists')
+    # NOTE(sigmavirus24): PyPI presently returns a 400 status code with the
+    # error message in the reason attribute. Other implementations return a
+    # 409 status code. We only want to skip an upload if:
+    # 1. The user has told us to skip existing packages (skip_existing is
+    #    True) AND
+    # 2. a) The response status code is 409 OR
+    # 2. b) The response status code is 400 AND it has a reason that matches
+    #       what we expect PyPI to return to us.
+    return (skip_existing and (response.status_code == 409 or
+            (response.status_code == 400 and response.reason.startswith(msg))))
 
 
 def upload(dists, repository, sign, identity, username, password, comment,
@@ -95,6 +105,11 @@ def upload(dists, repository, sign, identity, username, password, comment,
     for filename in uploads:
         package = PackageFile.from_filename(filename, comment)
 
+        if repository.package_is_uploaded(package) and skip_existing:
+            print("  Skipping {0} because it appears to already exist".format(
+                package.basefilename))
+            continue
+
         signed_name = package.signed_basefilename
         if signed_name in signatures:
             package.add_gpg_signature(signatures[signed_name], signed_name)
@@ -113,12 +128,6 @@ def upload(dists, repository, sign, identity, username, password, comment,
                  ' Aborting...').format(config["repository"],
                                         resp.headers["location"]))
 
-        # Otherwise, raise an HTTPError based on the status code.
-        if skip_upload(resp, skip_existing, package):
-            print("  Skipping {0} because it appears to already exist".format(
-                package.basefilename))
-            continue
-
         resp.raise_for_status()
 
     # Bug 28. Try to silence a ResourceWarning by clearing the connection
